@@ -14,7 +14,6 @@ from operator import attrgetter
 from warnings import warn
 
 from future.utils import raise_from, raise_with_traceback
-from six import iteritems, iterkeys, string_types
 
 from cobra.core.configuration import Configuration
 from cobra.core.gene import Gene, ast2str, eval_gpr, parse_gpr
@@ -86,9 +85,6 @@ class Reaction(Object):
         # this reaction.
         self._metabolites = {}
 
-        # The set of compartments that partaking metabolites are in.
-        self._compartments = None
-
         # self.model is None or refers to the cobra.Model that
         # contains self
         self._model = None
@@ -243,17 +239,6 @@ class Reaction(Object):
     @lower_bound.setter
     @resettable
     def lower_bound(self, value):
-        if self._upper_bound < value:
-            warn(
-                "You are constraining the reaction '{}' to a fixed flux "
-                "value of {}. Did you intend to do this? We are planning to "
-                "remove this behavior in a future release. Please let us "
-                "know your opinion at "
-                "https://github.com/opencobra/cobrapy/issues/793."
-                "".format(self.id, value),
-                DeprecationWarning,
-            )
-            self.upper_bound = value
         # Validate bounds before setting them.
         self._check_bounds(value, self._upper_bound)
         self._lower_bound = value
@@ -276,17 +261,6 @@ class Reaction(Object):
     @upper_bound.setter
     @resettable
     def upper_bound(self, value):
-        if self._lower_bound > value:
-            warn(
-                "You are constraining the reaction '{}' to a fixed flux "
-                "value of {}. Did you intend to do this? We are planning to "
-                "remove this behavior in a future release. Please let us "
-                "know your opinion at "
-                "https://github.com/opencobra/cobrapy/issues/793."
-                "".format(self.id, value),
-                DeprecationWarning,
-            )
-            self.lower_bound = value
         # Validate bounds before setting them.
         self._check_bounds(self._lower_bound, value)
         self._upper_bound = value
@@ -725,7 +699,7 @@ class Reaction(Object):
         bounds are swapped.
         """
         self._metabolites = {
-            met: value * coefficient for met, value in iteritems(self._metabolites)
+            met: value * coefficient for met, value in self._metabolites.items()
         }
 
         if coefficient < 0:
@@ -749,12 +723,12 @@ class Reaction(Object):
     @property
     def reactants(self):
         """Return a list of reactants for the reaction."""
-        return [k for k, v in iteritems(self._metabolites) if v < 0]
+        return [k for k, v in self._metabolites.items() if v < 0]
 
     @property
     def products(self):
         """Return a list of products for the reaction"""
-        return [k for k, v in iteritems(self._metabolites) if v >= 0]
+        return [k for k, v in self._metabolites.items() if v >= 0]
 
     def get_coefficient(self, metabolite_id):
         """
@@ -812,7 +786,7 @@ class Reaction(Object):
         new_metabolites = []
         _id_to_metabolites = dict([(x.id, x) for x in self._metabolites])
 
-        for metabolite, coefficient in iteritems(metabolites_to_add):
+        for metabolite, coefficient in metabolites_to_add.items():
 
             # Make sure metabolites being added belong to the same model, or
             # else copy them.
@@ -843,7 +817,7 @@ class Reaction(Object):
                         else:
                             # do we want to handle creation here?
                             raise e
-                elif isinstance(metabolite, string_types):
+                elif isinstance(metabolite, str):
                     # if we want to handle creation, this should be changed
                     raise ValueError(
                         "Reaction '%s' does not belong to a "
@@ -892,7 +866,7 @@ class Reaction(Object):
                 # Reset them with add_metabolites
                 mets_to_reset = {
                     key: old_coefficients[model.metabolites.get_by_any(key)[0]]
-                    for key in iterkeys(metabolites_to_add)
+                    for key in metabolites_to_add.keys()
                 }
 
                 context(
@@ -934,7 +908,7 @@ class Reaction(Object):
 
         """
         self.add_metabolites(
-            {k: -v for k, v in iteritems(metabolites)},
+            {k: -v for k, v in metabolites.items()},
             combine=combine,
             reversibly=reversibly,
         )
@@ -986,26 +960,22 @@ class Reaction(Object):
         This should be empty for balanced reactions.
         """
         reaction_element_dict = defaultdict(int)
-        for metabolite, coefficient in iteritems(self._metabolites):
+        for metabolite, coefficient in self._metabolites.items():
             if metabolite.charge is not None:
                 reaction_element_dict["charge"] += coefficient * metabolite.charge
             if metabolite.elements is None:
                 raise ValueError("No elements found in metabolite %s" % metabolite.id)
-            for element, amount in iteritems(metabolite.elements):
+            for element, amount in metabolite.elements.items():
                 reaction_element_dict[element] += coefficient * amount
         # filter out 0 values
-        return {k: v for k, v in iteritems(reaction_element_dict) if v != 0}
+        return {k: v for k, v in reaction_element_dict.items() if v != 0}
 
     @property
     def compartments(self):
         """lists compartments the metabolites are in"""
-        if self._compartments is None:
-            self._compartments = {
-                met.compartment
-                for met in self._metabolites
-                if met.compartment is not None
-            }
-        return self._compartments
+        return {
+            met.compartment for met in self._metabolites if met.compartment is not None
+        }
 
     def get_compartments(self):
         """lists compartments the metabolites are in"""
