@@ -33,17 +33,15 @@ import datetime
 import logging
 import os
 import re
-import traceback
 from collections import defaultdict, namedtuple
 from copy import deepcopy
 from sys import platform
 
 import libsbml
-from six import iteritems, raise_from, string_types
 
 import cobra
 from cobra.core import Gene, Group, Metabolite, Model, Reaction
-from cobra.core.gene import parse_gpr
+from cobra.core.gene import ast2str, parse_gpr
 from cobra.manipulation.validate import check_metabolite_compartment_formula
 from cobra.util.solver import linear_reaction_coefficients, set_objective
 
@@ -55,7 +53,7 @@ except ImportError:
 
 
 class CobraSBMLError(Exception):
-    """ SBML error class. """
+    """SBML error class."""
 
     pass
 
@@ -116,7 +114,7 @@ def _escape_non_alphanum(nonASCII):
 
 
 def _number_to_chr(numberStr):
-    """converts an ascii number to a character """
+    """converts an ascii number to a character"""
     return chr(int(numberStr.group(1)))
 
 
@@ -262,7 +260,7 @@ def read_sbml_model(filename, number=float, f_replace=F_REPLACE, **kwargs):
             "\nIf the model is valid and cannot be read please open an issue "
             "at https://github.com/opencobra/cobrapy/issues ."
         )
-        raise_from(cobra_error, original_error)
+        raise cobra_error from original_error
 
 
 def _get_doc_from_filename(filename):
@@ -276,7 +274,7 @@ def _get_doc_from_filename(filename):
     -------
     libsbml.SBMLDocument
     """
-    if isinstance(filename, string_types):
+    if isinstance(filename, str):
         if ("win" in platform) and (len(filename) < 260) and os.path.exists(filename):
             # path (win)
             doc = libsbml.readSBMLFromFile(
@@ -744,17 +742,8 @@ def _sbml_to_model(
                 if f_replace and F_GENE in f_replace:
                     gpr = " ".join(f_replace[F_GENE](t) for t in gpr.split(" "))
 
-        # remove outside parenthesis, if any
-        if gpr.startswith("(") and gpr.endswith(")"):
-            try:
-                parse_gpr(gpr[1:-1].strip())
-                gpr = gpr[1:-1].strip()
-            except (SyntaxError, TypeError) as e:
-                LOGGER.warning(
-                    "Removing parenthesis from gpr %s leads to "
-                    "an error, so keeping parenthesis",
-                    gpr,
-                )
+        # remove outside parenthesis and format into standard form
+        gpr = ast2str(parse_gpr(gpr)[0])
 
         cobra_reaction.gene_reaction_rule = gpr
 
@@ -963,7 +952,7 @@ def write_sbml_model(cobra_model, filename, f_replace=F_REPLACE, **kwargs):
     """
     doc = _model_to_sbml(cobra_model, f_replace=f_replace, **kwargs)
 
-    if isinstance(filename, string_types):
+    if isinstance(filename, str):
         # write to path
         libsbml.writeSBMLToFile(doc, filename)
 
@@ -1090,7 +1079,7 @@ def _model_to_sbml(cobra_model, f_replace=None, units=True):
     # Compartments
     # FIXME: use first class compartment model (and write notes & annotations)
     #     (https://github.com/opencobra/cobrapy/issues/811)
-    for cid, name in iteritems(cobra_model.compartments):
+    for cid, name in cobra_model.compartments.items():
         compartment = model.createCompartment()  # type: libsbml.Compartment
         compartment.setId(cid)
         compartment.setName(name)
@@ -1158,7 +1147,7 @@ def _model_to_sbml(cobra_model, f_replace=None, units=True):
         _sbase_notes_dict(reaction, cobra_reaction.notes)
 
         # stoichiometry
-        for metabolite, stoichiometry in iteritems(cobra_reaction._metabolites):
+        for metabolite, stoichiometry in cobra_reaction._metabolites.items():
             sid = metabolite.id
             if f_replace and F_SPECIE_REV in f_replace:
                 sid = f_replace[F_SPECIE_REV](sid)
@@ -1203,19 +1192,15 @@ def _model_to_sbml(cobra_model, f_replace=None, units=True):
         # GPR
         gpr = cobra_reaction.gene_reaction_rule
         if gpr is not None and len(gpr) > 0:
-
+            # we will parse the GPR to format them correctly for libSBML
+            # avoids cases where a GPR can be parsed by cobrapy but not libSBML
+            tree, gids = parse_gpr(gpr)
             # replace ids in string
             if f_replace and F_GENE_REV in f_replace:
-                gpr = gpr.replace("(", "( ")
-                gpr = gpr.replace(")", " )")
-                tokens = gpr.split()
-
-                for k in range(len(tokens)):
-                    if tokens[k] not in ["and", "or", "(", ")"]:
-                        tokens[k] = f_replace[F_GENE_REV](tokens[k])
-                gpr_new = " ".join(tokens)
+                idmap = {gid: f_replace[F_GENE_REV](gid) for gid in gids}
+                gpr_new = ast2str(tree, names=idmap)
             else:
-                gpr_new = gpr
+                gpr_new = ast2str(tree)
 
             gpa = (
                 r_fbc.createGeneProductAssociation()
@@ -1544,7 +1529,7 @@ def _parse_annotations(sbase):
                 provider, identifier = data
 
             if provider in annotation:
-                if isinstance(annotation[provider], string_types):
+                if isinstance(annotation[provider], str):
                     annotation[provider] = [annotation[provider]]
                 # FIXME: use a list
                 if identifier not in annotation[provider]:
@@ -1610,12 +1595,12 @@ def _sbase_annotations(sbase, annotation):
         # handling of non-string annotations (e.g. integers)
         if isinstance(value, (float, int)):
             value = str(value)
-        if isinstance(value, string_types):
+        if isinstance(value, str):
             annotation_data[key] = [("is", value)]
 
     for key, value in annotation_data.items():
         for idx, item in enumerate(value):
-            if isinstance(item, string_types):
+            if isinstance(item, str):
                 value[idx] = ("is", item)
 
     # set metaId
@@ -1623,7 +1608,7 @@ def _sbase_annotations(sbase, annotation):
     sbase.setMetaId(meta_id)
 
     # rdf_items = []
-    for provider, data in iteritems(annotation_data):
+    for provider, data in annotation_data.items():
 
         # set SBOTerm
         if provider in ["SBO", "sbo"]:
