@@ -1,14 +1,10 @@
-# -*- coding: utf-8 -*-
-
-"""Test functions of reaction.py"""
-
-from __future__ import absolute_import
+"""Test functions of cobra.core.reaction ."""
 
 import warnings
+from typing import Iterable
 
 import numpy as np
 import pytest
-import six
 
 from cobra.core import Configuration, Metabolite, Model, Reaction
 
@@ -17,7 +13,8 @@ config = Configuration()
 stable_optlang = ["glpk", "cplex", "gurobi"]
 
 
-def test_gpr():
+def test_gpr() -> None:
+    """Test GPR evaluation."""
     model = Model()
     reaction = Reaction("test")
 
@@ -52,7 +49,8 @@ def test_gpr():
         assert len(reaction.genes) == 1
 
 
-def test_gpr_modification(model):
+def test_gpr_modification(model: Model) -> None:
+    """Test GPR manipulations."""
     reaction = model.reactions.get_by_id("PGI")
     old_gene = list(reaction.genes)[0]
     new_gene = model.genes.get_by_id("s0001")
@@ -76,7 +74,8 @@ def test_gpr_modification(model):
     assert reaction.gene_name_reaction_rule == fake_gene.name
 
 
-def test_gene_knock_out(model):
+def test_gene_knock_out(model: Model) -> None:
+    """Test gene knockout effect on reaction."""
     rxn = Reaction("rxn")
     rxn.add_metabolites({Metabolite("A"): -1, Metabolite("B"): 1})
     rxn.gene_reaction_rule = "A2B1 or A2B2 and A2B3"
@@ -96,17 +95,20 @@ def test_gene_knock_out(model):
     assert not model.reactions.rxn.functional
 
 
-def test_str():
+def test_str() -> None:
+    """Test `str` output for a reaction."""
     rxn = Reaction("rxn")
     rxn.add_metabolites({Metabolite("A"): -1, Metabolite("B"): 1})
     assert str(rxn) == "rxn: A --> B"
 
 
-def test_str_from_model(model):
+def test_str_from_model(model: Model) -> None:
+    """Test `str` output for a reaction associated with a model."""
     assert model.reactions[0].__str__().startswith("ACALD")
 
 
-def test_add_metabolite_from_solved_model(solved_model):
+def test_add_metabolite_from_solved_model(solved_model: Model) -> None:
+    """Test metabolite addition to a reaction from a solved model."""
     solution, model = solved_model
     pgi_reaction = model.reactions.PGI
     test_met = model.metabolites[0]
@@ -158,24 +160,26 @@ def test_add_metabolite_from_solved_model(solved_model):
 
 
 @pytest.mark.parametrize("solver", stable_optlang)
-def test_add_metabolite_benchmark(model, benchmark, solver):
+def test_add_metabolite_benchmark(model: Model, benchmark, solver: Iterable) -> None:
+    """Benchmark metabolite addition to a reaction associated with a model."""
     reaction = model.reactions.get_by_id("PGI")
     many_metabolites = dict((m, 1) for m in model.metabolites[0:50])
 
     def add_remove_metabolite():
         reaction.add_metabolites(many_metabolites)
         if not getattr(model, "solver", None):
-            solver_dict[solver].create_problem(model)
-        for m, c in many_metabolites.items():
+            stable_optlang[solver].create_problem(model)
+        for met in many_metabolites:
             try:
-                reaction.subtract_metabolites({m: reaction.get_coefficient(m)})
+                reaction.subtract_metabolites({met: reaction.get_coefficient(met)})
             except KeyError:
                 pass
 
     benchmark(add_remove_metabolite)
 
 
-def test_add_metabolite(model):
+def test_add_metabolite(model: Model) -> None:
+    """Test metabolite addition to a reaction from an unsolved model."""
     with pytest.raises(ValueError):
         model.add_metabolites(Metabolite())
     with model:
@@ -223,20 +227,25 @@ def test_add_metabolite(model):
 
 
 @pytest.mark.parametrize("solver", stable_optlang)
-def test_subtract_metabolite_benchmark(model, benchmark, solver):
+def test_subtract_metabolite_benchmark(
+    model: Model, benchmark, solver: Iterable
+) -> None:
+    """Benchmark metabolite deletion from a reaction."""
     benchmark(test_subtract_metabolite, model, solver)
 
 
 @pytest.mark.parametrize("solver", stable_optlang)
-def test_subtract_metabolite(model, solver):
+def test_subtract_metabolite(model: Model, solver: Iterable) -> None:
+    """Test metabolite deletion from a reaction associated with an unsolved model."""
     reaction = model.reactions.get_by_id("PGI")
     reaction.subtract_metabolites(reaction.metabolites)
     if not getattr(model, "solver", None):
-        solver_dict[solver].create_problem(model)
+        stable_optlang[solver].create_problem(model)
         assert len(reaction.metabolites) == 0
 
 
-def test_mass_balance(model):
+def test_mass_balance(model: Model) -> None:
+    """Test mass balance of metabolites of a reaction."""
     reaction = model.reactions.get_by_id("PGI")
     # Should be balanced now
     assert len(reaction.check_mass_balance()) == 0
@@ -247,7 +256,8 @@ def test_mass_balance(model):
     assert imbalance["H"] == 1
 
 
-def test_build_from_string(model):
+def test_build_from_string(model: Model) -> None:
+    """Test reaction building from string evaluation."""
     m = len(model.metabolites)
     pgi = model.reactions.get_by_id("PGI")
     old_bounds = pgi.bounds
@@ -291,13 +301,15 @@ def test_build_from_string(model):
         assert pgi.bounds == (0, 1000)
 
 
-def test_bounds_setter(model):
+def test_bounds_setter(model: Model) -> None:
+    """Test reaction bounds setter."""
     rxn = model.reactions.get_by_id("PGI")
     with pytest.raises(ValueError):
         rxn.bounds = (1, 0)
 
 
-def test_copy(model):
+def test_copy(model: Model) -> None:
+    """Test reaction copying."""
     PGI = model.reactions.PGI
     copied = PGI.copy()
     assert PGI is not copied
@@ -313,7 +325,8 @@ def test_copy(model):
     assert len(model.get_associated_groups(copied.id)) == 0
 
 
-def test_iadd(model):
+def test_iadd(model: Model) -> None:
+    """Test in-place addition of reaction."""
     PGI = model.reactions.PGI
     EX_h2o = model.reactions.EX_h2o_e
     original_PGI_gpr = PGI.gene_reaction_rule
@@ -341,7 +354,8 @@ def test_iadd(model):
     assert len(model.reactions.ACKr.genes) == 5
 
 
-def test_add(model):
+def test_add(model: Model) -> None:
+    """Test reaction addition to model."""
     # Not in place addition should work on a copy
     new = model.reactions.PGI + model.reactions.EX_h2o_e
     assert new._model is not model
@@ -357,24 +371,28 @@ def test_add(model):
         assert gene.model is not model
 
 
-def test_radd(model):
+def test_radd(model: Model) -> None:
+    """Test __radd__ for a reaction."""
     new = sum([model.reactions.PGI, model.reactions.EX_h2o_e])
     assert new._model is not model
     assert len(new.metabolites) == 3
 
 
-def test_mul(model):
+def test_mul(model: Model) -> None:
+    """Test scalar multiplication of factors with a reaction."""
     new = model.reactions.PGI * 2
     assert set(new.metabolites.values()) == {-2, 2}
 
 
-def test_sub(model):
+def test_sub(model: Model) -> None:
+    """Test reaction subtraction."""
     new = model.reactions.PGI - model.reactions.EX_h2o_e
     assert new._model is not model
     assert len(new.metabolites) == 3
 
 
-def test_removal_from_model_retains_bounds(model):
+def test_removal_from_model_retains_bounds(model: Model) -> None:
+    """Test reaction removal from a model, retains its bounds."""
     model_cp = model.copy()
     reaction = model_cp.reactions.ACALD
     assert reaction.model == model_cp
@@ -390,7 +408,8 @@ def test_removal_from_model_retains_bounds(model):
     assert reaction._upper_bound == 1000.0
 
 
-def test_set_bounds_scenario_1(model):
+def test_set_bounds_scenario_1(model: Model) -> None:
+    """Test reaction bounds setting for a scenario."""
     acald_reaction = model.reactions.ACALD
     assert acald_reaction.lower_bound == -1000.0
     assert acald_reaction.upper_bound == 1000.0
@@ -398,7 +417,10 @@ def test_set_bounds_scenario_1(model):
     assert acald_reaction.forward_variable.ub == 1000.0
     assert acald_reaction.reverse_variable.lb == 0
     assert acald_reaction.reverse_variable.ub == 1000.0
-    acald_reaction.upper_bound = acald_reaction.lower_bound - 100
+    acald_reaction.bounds = (
+        acald_reaction.lower_bound - 100,
+        acald_reaction.lower_bound - 100,
+    )
     assert acald_reaction.lower_bound == -1100.0
     assert acald_reaction.upper_bound == -1100.0
     assert acald_reaction.forward_variable.lb == 0
@@ -414,57 +436,8 @@ def test_set_bounds_scenario_1(model):
     assert acald_reaction.reverse_variable.ub == 1100.0
 
 
-def test_set_bounds_scenario_3(model):
-    reac = model.reactions.ACALD
-    reac.upper_bound = -10
-    reac.lower_bound = -10
-    assert reac.lower_bound == -10
-    assert reac.upper_bound == -10
-    reac.lower_bound = -9
-    assert reac.lower_bound == -9
-    assert reac.upper_bound == -9
-    reac.lower_bound = 2
-    assert reac.lower_bound == 2
-    assert reac.upper_bound == 2
-    reac.upper_bound = -10
-    assert reac.lower_bound == -10
-    assert reac.upper_bound == -10
-    reac.upper_bound = -11
-    assert reac.lower_bound == -11
-    assert reac.upper_bound == -11
-    reac.upper_bound = 2
-    assert reac.lower_bound == -11
-    assert reac.upper_bound == 2
-
-
-def test_set_bounds_scenario_4(model):
-    reac = model.reactions.ACALD
-    reac.lower_bound = reac.upper_bound = 0
-    reac.lower_bound = 2
-    assert reac.lower_bound == 2
-    assert reac.upper_bound == 2
-    assert reac.forward_variable.lb == 2
-    assert reac.forward_variable.ub == 2
-    reac.knock_out()
-    reac.upper_bound = -2
-    assert reac.lower_bound == -2
-    assert reac.upper_bound == -2
-    assert reac.reverse_variable.lb == 2
-    assert reac.reverse_variable.ub == 2
-
-
-def test_set_upper_before_lower_bound_to_0(model):
-    model.reactions.GAPD.upper_bound = 0
-    model.reactions.GAPD.lower_bound = 0
-    assert model.reactions.GAPD.lower_bound == 0
-    assert model.reactions.GAPD.upper_bound == 0
-    assert model.reactions.GAPD.forward_variable.lb == 0
-    assert model.reactions.GAPD.forward_variable.ub == 0
-    assert model.reactions.GAPD.reverse_variable.lb == 0
-    assert model.reactions.GAPD.reverse_variable.ub == 0
-
-
-def test_set_bounds_scenario_2(model):
+def test_set_bounds_scenario_2(model: Model) -> None:
+    """Test reaction bounds setting for a scenario."""
     acald_reaction = model.reactions.ACALD
     assert acald_reaction.lower_bound == -1000.0
     assert acald_reaction.upper_bound == 1000.0
@@ -472,7 +445,10 @@ def test_set_bounds_scenario_2(model):
     assert acald_reaction.forward_variable.ub == 1000.0
     assert acald_reaction.reverse_variable.lb == 0
     assert acald_reaction.reverse_variable.ub == 1000.0
-    acald_reaction.lower_bound = acald_reaction.upper_bound + 100
+    acald_reaction.bounds = (
+        acald_reaction.upper_bound + 100,
+        acald_reaction.upper_bound + 100,
+    )
     assert acald_reaction.lower_bound == 1100.0
     assert acald_reaction.upper_bound == 1100.0
     assert acald_reaction.forward_variable.lb == 1100.0
@@ -488,7 +464,58 @@ def test_set_bounds_scenario_2(model):
     assert acald_reaction.reverse_variable.ub == 100
 
 
-def test_change_bounds(model):
+def test_set_bounds_scenario_3(model: Model) -> None:
+    """Test reaction bounds setting for a scenario."""
+    reac = model.reactions.ACALD
+    reac.bounds = (-10, -10)
+    assert reac.lower_bound == -10
+    assert reac.upper_bound == -10
+    reac.bounds = (-9, -9)
+    assert reac.lower_bound == -9
+    assert reac.upper_bound == -9
+    reac.bounds = (2, 2)
+    assert reac.lower_bound == 2
+    assert reac.upper_bound == 2
+    reac.bounds = (-10, -10)
+    assert reac.lower_bound == -10
+    assert reac.upper_bound == -10
+    reac.bounds = (-11, -11)
+    assert reac.lower_bound == -11
+    assert reac.upper_bound == -11
+    reac.upper_bound = 2
+    assert reac.lower_bound == -11
+    assert reac.upper_bound == 2
+
+
+def test_set_bounds_scenario_4(model: Model) -> None:
+    """Test reaction bounds setting for a scenario."""
+    reac = model.reactions.ACALD
+    reac.bounds = (2, 2)
+    assert reac.lower_bound == 2
+    assert reac.upper_bound == 2
+    assert reac.forward_variable.lb == 2
+    assert reac.forward_variable.ub == 2
+    reac.knock_out()
+    reac.bounds = (-2, -2)
+    assert reac.lower_bound == -2
+    assert reac.upper_bound == -2
+    assert reac.reverse_variable.lb == 2
+    assert reac.reverse_variable.ub == 2
+
+
+def test_set_upper_before_lower_bound_to_0(model: Model) -> None:
+    """Test reaction bounds setting to zero."""
+    model.reactions.GAPD.bounds = (0, 0)
+    assert model.reactions.GAPD.lower_bound == 0
+    assert model.reactions.GAPD.upper_bound == 0
+    assert model.reactions.GAPD.forward_variable.lb == 0
+    assert model.reactions.GAPD.forward_variable.ub == 0
+    assert model.reactions.GAPD.reverse_variable.lb == 0
+    assert model.reactions.GAPD.reverse_variable.ub == 0
+
+
+def test_change_bounds(model: Model) -> None:
+    """Test reaction bounds change."""
     reac = model.reactions.ACALD
     reac.bounds = (2, 2)
     assert reac.lower_bound == 2
@@ -501,7 +528,8 @@ def test_change_bounds(model):
     assert reac.upper_bound == 2
 
 
-def test_make_irreversible(model):
+def test_make_irreversible(model: Model) -> None:
+    """Test reaction irreversibility."""
     acald_reaction = model.reactions.ACALD
     assert acald_reaction.lower_bound == -1000.0
     assert acald_reaction.upper_bound == 1000.0
@@ -525,7 +553,8 @@ def test_make_irreversible(model):
     assert acald_reaction.reverse_variable.ub == 100
 
 
-def test_make_reversible(model):
+def test_make_reversible(model: Model) -> None:
+    """Test reaction reversibility."""
     pfk_reaction = model.reactions.PFK
     assert pfk_reaction.lower_bound == 0.0
     assert pfk_reaction.upper_bound == 1000.0
@@ -549,7 +578,8 @@ def test_make_reversible(model):
     assert pfk_reaction.reverse_variable.ub == 0
 
 
-def test_make_irreversible_irreversible_to_the_other_side(model):
+def test_make_irreversible_irreversible_to_the_other_side(model: Model) -> None:
+    """Test reaction irreversibility to irreversibility."""
     pfk_reaction = model.reactions.PFK
     assert pfk_reaction.lower_bound == 0.0
     assert pfk_reaction.upper_bound == 1000.0
@@ -557,7 +587,7 @@ def test_make_irreversible_irreversible_to_the_other_side(model):
     assert pfk_reaction.forward_variable.ub == 1000.0
     assert pfk_reaction.reverse_variable.lb == 0
     assert pfk_reaction.reverse_variable.ub == 0
-    pfk_reaction.upper_bound = -100.0
+    pfk_reaction.bounds = (-100.0, -100.0)
     assert pfk_reaction.forward_variable.lb == 0
     assert pfk_reaction.forward_variable.ub == 0
     assert pfk_reaction.reverse_variable.lb == 100
@@ -571,11 +601,11 @@ def test_make_irreversible_irreversible_to_the_other_side(model):
     assert pfk_reaction.reverse_variable.ub == 1000.0
 
 
-def test_make_lhs_irreversible_reversible(model):
+def test_make_lhs_irreversible_reversible(model: Model) -> None:
+    """Test reaction LHS irreversibility to reversibility."""
     rxn = Reaction("test")
     rxn.add_metabolites({model.metabolites[0]: -1.0, model.metabolites[1]: 1.0})
-    rxn.lower_bound = -1000.0
-    rxn.upper_bound = -100
+    rxn.bounds = (-1000.0, -100)
     model.add_reaction(rxn)
     assert rxn.lower_bound == -1000.0
     assert rxn.upper_bound == -100.0
@@ -592,7 +622,8 @@ def test_make_lhs_irreversible_reversible(model):
     assert rxn.reverse_variable.ub == 1000.0
 
 
-def test_model_less_reaction(model):
+def test_model_less_reaction(model: Model) -> None:
+    """Test model without reactions."""
     model.slim_optimize()
     for reaction in model.reactions:
         assert isinstance(reaction.flux, float)
@@ -605,16 +636,16 @@ def test_model_less_reaction(model):
             reaction.reduced_cost
 
 
-def test_knockout(model):
+def test_knockout(model: Model) -> None:
+    """Test reaction knockouts."""
     original_bounds = dict()
     for reaction in model.reactions:
         original_bounds[reaction.id] = (reaction.lower_bound, reaction.upper_bound)
         reaction.knock_out()
         assert reaction.lower_bound == 0
         assert reaction.upper_bound == 0
-    for k, (lb, ub) in six.iteritems(original_bounds):
-        model.reactions.get_by_id(k).lower_bound = lb
-        model.reactions.get_by_id(k).upper_bound = ub
+    for k, (lb, ub) in original_bounds.items():
+        model.reactions.get_by_id(k).bounds = (lb, ub)
     for reaction in model.reactions:
         assert reaction.lower_bound == original_bounds[reaction.id][0]
         assert reaction.upper_bound == original_bounds[reaction.id][1]
@@ -629,14 +660,16 @@ def test_knockout(model):
         assert reaction.upper_bound == original_bounds[reaction.id][1]
 
 
-def test_reaction_without_model():
+def test_reaction_without_model() -> None:
+    """Test reaction without model association."""
     r = Reaction("blub")
     assert r.flux_expression is None
     assert r.forward_variable is None
     assert r.reverse_variable is None
 
 
-def test_weird_left_to_right_reaction_issue(tiny_toy_model):
+def test_weird_left_to_right_reaction_issue(tiny_toy_model: Model) -> None:
+    """Test absurd left to right reaction."""
     d1 = tiny_toy_model.reactions.get_by_id("ex1")
     assert not d1.reversibility
     assert d1.lower_bound == -1000
@@ -655,7 +688,8 @@ def test_weird_left_to_right_reaction_issue(tiny_toy_model):
     assert d1._upper_bound == 0
 
 
-def test_one_left_to_right_reaction_set_positive_ub(tiny_toy_model):
+def test_one_left_to_right_reaction_set_positive_ub(tiny_toy_model: Model) -> None:
+    """Test left to right reaction with positive upper bound."""
     d1 = tiny_toy_model.reactions.get_by_id("ex1")
     assert d1.reverse_variable.lb == 0
     assert d1.reverse_variable.ub == 1000
@@ -676,7 +710,8 @@ def test_one_left_to_right_reaction_set_positive_ub(tiny_toy_model):
     assert d1.upper_bound == 0.1
 
 
-def test_irrev_reaction_set_negative_lb(model):
+def test_irrev_reaction_set_negative_lb(model: Model) -> None:
+    """Test reaction irreversibility with negative lower bound."""
     assert not model.reactions.PFK.reversibility
     assert model.reactions.PFK.lower_bound == 0
     assert model.reactions.PFK.upper_bound == 1000.0
@@ -693,7 +728,8 @@ def test_irrev_reaction_set_negative_lb(model):
     assert model.reactions.PFK.reverse_variable.ub == 1000
 
 
-def test_twist_irrev_right_to_left_reaction_to_left_to_right(model):
+def test_twist_irrev_right_to_left_reaction_to_left_to_right(model: Model) -> None:
+    """Test irreversibility reversal from right to left to left to right."""
     assert not model.reactions.PFK.reversibility
     assert model.reactions.PFK.lower_bound == 0
     assert model.reactions.PFK.upper_bound == 1000.0
@@ -701,8 +737,7 @@ def test_twist_irrev_right_to_left_reaction_to_left_to_right(model):
     assert model.reactions.PFK.forward_variable.ub == 1000.0
     assert model.reactions.PFK.reverse_variable.lb == 0
     assert model.reactions.PFK.reverse_variable.ub == 0
-    model.reactions.PFK.lower_bound = -1000
-    model.reactions.PFK.upper_bound = 0
+    model.reactions.PFK.bounds = (-1000, 0)
     assert model.reactions.PFK.lower_bound == -1000
     assert model.reactions.PFK.upper_bound == 0
     assert model.reactions.PFK.forward_variable.lb == 0
@@ -711,21 +746,24 @@ def test_twist_irrev_right_to_left_reaction_to_left_to_right(model):
     assert model.reactions.PFK.reverse_variable.ub == 1000
 
 
-def test_set_lb_higher_than_ub_sets_ub_to_new_lb(model):
+def test_set_lb_higher_than_ub_sets_ub_to_new_lb(model: Model) -> None:
+    """Test lower bound > upper bound makes upper bound to new lower bound."""
     for reaction in model.reactions:
         assert reaction.lower_bound <= reaction.upper_bound
-        reaction.lower_bound = reaction.upper_bound + 100
+        reaction.bounds = (reaction.upper_bound + 100, reaction.upper_bound + 100)
         assert reaction.lower_bound == reaction.upper_bound
 
 
-def test_set_ub_lower_than_lb_sets_lb_to_new_ub(model):
+def test_set_ub_lower_than_lb_sets_lb_to_new_ub(model: Model) -> None:
+    """Test upper bound < lower bound makes lower bound to new upper bound."""
     for reaction in model.reactions:
         assert reaction.lower_bound <= reaction.upper_bound
-        reaction.upper_bound = reaction.lower_bound - 100
+        reaction.bounds = (reaction.lower_bound - 100, reaction.lower_bound - 100)
         assert reaction.lower_bound == reaction.upper_bound
 
 
-def test_add_metabolites_combine_true(model):
+def test_add_metabolites_combine_true(model: Model) -> None:
+    """Test metabolite addition to reaction (with combine = True)."""
     test_metabolite = Metabolite("test")
     for reaction in model.reactions:
         reaction.add_metabolites({test_metabolite: -66}, combine=True)
@@ -762,7 +800,8 @@ def test_add_metabolites_combine_true(model):
 
 
 @pytest.mark.xfail(reason="non-deterministic test")
-def test_add_metabolites_combine_false(model):
+def test_add_metabolites_combine_false(model: Model) -> None:
+    """Test metabolite addition to reaction (with combine = False)."""
     test_metabolite = Metabolite("test")
     for reaction in model.reactions:
         reaction.add_metabolites({test_metabolite: -66}, combine=False)
@@ -784,7 +823,8 @@ def test_add_metabolites_combine_false(model):
         )
 
 
-def test_reaction_imul(model):
+def test_reaction_imul(model: Model) -> None:
+    """Test in-place scalar factor multiplication to reaction."""
     with model:
         model.reactions.EX_glc__D_e *= 100
         assert (
@@ -830,7 +870,8 @@ def test_reaction_imul(model):
 #            ).as_coefficients_dict()
 
 
-def test_remove_from_model(model):
+def test_remove_from_model(model: Model) -> None:
+    """Test reaction removal from model."""
     pgi = model.reactions.PGI
     g6p = model.metabolites.g6p_c
     pgi_flux = model.optimize().fluxes["PGI"]
@@ -854,7 +895,8 @@ def test_remove_from_model(model):
     assert np.isclose(pgi_flux, model.optimize().fluxes["PGI"])
 
 
-def test_change_id_is_reflected_in_solver(model):
+def test_change_id_is_reflected_in_solver(model: Model) -> None:
+    """Test reaction ID change reflection in solver."""
     for i, reaction in enumerate(model.reactions):
         old_reaction_id = reaction.id
         assert model.variables[old_reaction_id].name == old_reaction_id
@@ -869,5 +911,14 @@ def test_change_id_is_reflected_in_solver(model):
         assert name == reaction.id
 
 
-def test_repr_html_(model):
+def test_repr_html_(model: Model) -> None:
+    """Test __repr_html__ functionality."""
     assert "<table>" in model.reactions[0]._repr_html_()
+
+
+def test_compartment_changes(model: Model) -> None:
+    """Test reaction compartment change."""
+    rxn = model.reactions.EX_ac_e
+    assert rxn.reactants[0].compartment in rxn.compartments
+    rxn.reactants[0].compartment = "blub"
+    assert rxn.reactants[0].compartment in rxn.compartments
